# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-11-22 05:40
from __future__ import unicode_literals

import bee_django_richtext.custom_fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_wiki', '0007_auto_20190903_1846'),
    ]

    operations = [
        migrations.AlterField(
            model_name='topic',
            name='detail',
            field=bee_django_richtext.custom_fields.RichTextField(app_name='bee_django_wiki', emotion=False, image_max_size=2, img=False, model_name='Topic', text_min_length=5, undo_redo=False, verbose_name='\u8be6\u60c5'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-09-03 08:45
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_wiki', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='topic',
            old_name='classfify',
            new_name='classify',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.contrib.auth.models import Group
from bee_django_richtext.custom_fields import RichTextField
# Create your models here.
class Classify(models.Model):
    name = models.CharField(max_length=180, verbose_name='分类')

    class Meta:
        app_label = 'bee_django_wiki'
        db_table = 'bee_django_wiki_classify'
        permissions = (
            ('view_all_classify', '可以查看分类'),
        )

    def __unicode__(self):
        return self.name


class Topic(models.Model):
    title = models.CharField(max_length=180, verbose_name='标题')
    detail = RichTextField(verbose_name='详情',app_name='bee_django_wiki', model_name='Topic',img=True)
    tag = models.CharField(max_length=180, verbose_name='标签', null=True, blank=True)
    classify = models.ForeignKey(Classify, verbose_name='分类', null=True, on_delete=models.SET_NULL)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    order_by = models.IntegerField(null=True, verbose_name='排序', default=0)
    # view_group = models.TextField(verbose_name='可观看的用户组', null=True)  # group id，开头结尾和中间用|分割，例如|1|2|
    view_group = models.ManyToManyField(Group,verbose_name='可观看的用户组（可多选）')
    class Meta:
        app_label = 'bee_django_wiki'
        db_table = 'bee_django_wiki_topic'
        ordering = ["-updated_at"]
        permissions = (
            ('view_all_topic', '查看所有主题'),  # 无视view_group
        )

    def __unicode__(self):
        return self.title


class TopicImage(models.Model):
    image = models.ImageField(verbose_name='图片', upload_to='wiki/%Y/%m/%d')
    class Meta:
        app_label = 'bee_django_wiki'
        db_table = 'bee_django_wiki_toipc_image'


# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-09-03 10:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_wiki', '0005_auto_20190903_1714'),
    ]

    operations = [
        migrations.CreateModel(
            name='TopicImage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.ImageField(upload_to='wiki/%Y/%m/%d', verbose_name='\u56fe\u7247')),
            ],
            options={
                'db_table': 'bee_django_wiki_toipc_image',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import render, HttpResponse
from django.conf import settings
from django.views.generic import ListView, DetailView, TemplateView
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.core.urlresolvers import reverse_lazy
from django.db.models import Q, Sum, Count
from django.contrib.auth import get_user_model
from django.views.decorators.csrf import csrf_exempt

from .models import Classify, Topic, TopicImage
from .forms import ClassifyForm, TopicForm, TopicSearchForm, TopicImageForm

User = get_user_model()


# Create your views here.
def test(request):
    return


# ========Classify==========
class ClassifyList(ListView):
    model = Classify
    template_name = 'bee_django_wiki/classify/list.html'
    context_object_name = 'classify_list'
    paginate_by = 20


class ClassifyCreate(CreateView):
    model = Classify
    form_class = ClassifyForm
    template_name = 'bee_django_wiki/classify/form.html'
    success_url = reverse_lazy("bee_django_wiki:classify_list")


class ClassifyUpdate(UpdateView):
    model = Classify
    form_class = ClassifyForm
    template_name = 'bee_django_wiki/classify/form.html'
    success_url = reverse_lazy("bee_django_wiki:classify_list")


class ClassifyDelete(DeleteView):
    model = Classify
    success_url = reverse_lazy('bee_django_wiki:classify_list')

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)


# ========Topic==========
class TopicList(ListView):
    model = Topic
    template_name = 'bee_django_wiki/topic/list.html'
    context_object_name = 'topic_list'
    paginate_by = 20

    def get_queryset(self):
        name = self.request.GET.get("name")
        classify_id = self.request.GET.get("classify_id")
        if self.request.user.has_perm("bee_django_wiki.view_all_topic"):
            queryset = Topic.objects.all()
        else:
            group_list = self.request.user.groups.all()
            queryset = Topic.objects.filter(view_group__topic__view_group__in=group_list).distinct()
        if not name in [None]:
            queryset = queryset.filter((Q(title__icontains=name) | Q(tag__icontains=name)))
        if not classify_id in [0, "0", None]:
            queryset = queryset.filter(classify_id=classify_id)
        return queryset

    # def search(self):
    #     name = self.request.GET.get("name")
    #     if not name in [None]:
    #         self.queryset =self.queryset.filter((Q(title__icontains=name) | Q(tag__icontains=name)))
    #     return self.queryset

    def get_context_data(self, **kwargs):
        context = super(TopicList, self).get_context_data(**kwargs)
        name = self.request.GET.get("name")
        context['search_form'] = TopicSearchForm({"name": name})
        context['classify_list'] = Classify.objects.all()
        return context


class TopicDetail(DetailView):
    model = Topic
    template_name = 'bee_django_wiki/topic/detail.html'
    context_object_name = 'topic'

    def get_object(self, queryset=None):
        if self.request.user.has_perm("bee_django_wiki.view_all_topic"):
            return Topic.objects.get(id=self.kwargs["pk"])
        else:
            group_list = self.request.user.groups.all()
            try:
                topic_list = Topic.objects.filter(id=self.kwargs["pk"], view_group__in=group_list).distinct()
                if topic_list.exists():
                    return topic_list[0]
            except Exception as e:
                # print e
                return None

        # return

    def get_context_data(self, **kwargs):
        context = super(TopicDetail, self).get_context_data(**kwargs)
        context['classify_list'] = Classify.objects.all()
        return context


class TopicCreate(CreateView):
    model = Topic
    form_class = TopicForm
    template_name = 'bee_django_wiki/topic/form.html'
    success_url = reverse_lazy("bee_django_wiki:topic_list")


class TopicUpdate(UpdateView):
    model = Topic
    form_class = TopicForm
    template_name = 'bee_django_wiki/topic/form.html'
    success_url = reverse_lazy("bee_django_wiki:topic_list")

    def get_success_url(self):
        return reverse_lazy("bee_django_wiki:topic_detail", kwargs=self.kwargs)


class TopicDelete(DeleteView):
    model = Topic
    success_url = reverse_lazy('bee_django_wiki:topic_list')

    def get(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)


@csrf_exempt
def upload_image(request):
    # 默认上传图片大小为2M
    if hasattr(settings, "WIKI_TOPIC_UPLOAD_MAXSIZE"):
        _max_size = settings.WIKI_TOPIC_UPLOAD_MAXSIZE
    else:
        _max_size = 2
    max_size = _max_size * 1024 * 1024
    if request.method == "POST":
        file = request.FILES.get('image')
        if file.size > max_size:
            return HttpResponse("error|图片大小不能超过" + _max_size.__str__() + "M!")

        # 保存图片。用户上传的图片，与用户的对应关系也保存到数据库中
        form = TopicImageForm(request.POST, request.FILES)
        if form.is_valid():
            topic_image = form.save(commit=True)
            return HttpResponse(topic_image.image.url)
        else:
            print(form.errors)
            return HttpResponse("error|文件存储错误")
    else:
        return HttpResponse("error|请求错误")



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-08-30 08:44
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Classify',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=180, verbose_name='\u5206\u7c7b')),
            ],
            options={
                'db_table': 'bee_django_wiki_classify',
                'permissions': (('view_all_classify', '\u53ef\u4ee5\u67e5\u770b\u5206\u7c7b'),),
            },
        ),
        migrations.CreateModel(
            name='Topic',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=180, verbose_name='\u6807\u9898')),
                ('detail', models.TextField(verbose_name='\u8be6\u60c5')),
                ('tag', models.CharField(max_length=180, null=True, verbose_name='\u6807\u7b7e')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('order_by', models.IntegerField(default=0, null=True, verbose_name='\u6392\u5e8f')),
                ('view_group', models.TextField(null=True, verbose_name='\u53ef\u89c2\u770b\u7684\u7528\u6237\u7ec4')),
                ('classfify', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bee_django_wiki.Classify', verbose_name='\u5206\u7c7b')),
            ],
            options={
                'ordering': ['-updated_at'],
                'db_table': 'bee_django_wiki_topic',
                'permissions': (('view_all_topic', '\u67e5\u770b\u6240\u6709\u4e3b\u9898'),),
            },
        ),
    ]



# -*- coding:utf-8 -*-
__author__ = 'bee'

from django import forms
from django.contrib.auth.models import Group
from .models import Classify, Topic,TopicImage


class ClassifyForm(forms.ModelForm):
    class Meta:
        model = Classify
        fields = ['name']


class TopicForm(forms.ModelForm):
    # view_group = forms.ModelChoiceField(queryset=Group.objects.all(), label='可观看用户组', required=False)
    class Meta:
        model = Topic
        fields = ["classify", "title", "view_group", 'tag', "detail"]


class TopicSearchForm(forms.Form):
    name = forms.CharField(label='', required=False, widget=forms.TextInput(attrs={'placeholder': '输入标题或标签进行搜索','style':'width:50%;height:34px;'}))

class TopicImageForm(forms.ModelForm):
    class Meta:
        model = TopicImage
        fields = ["image"]


# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-09-03 10:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_wiki', '0006_topicimage'),
    ]

    operations = [
        migrations.AlterField(
            model_name='topic',
            name='view_group',
            field=models.ManyToManyField(to='auth.Group', verbose_name='\u53ef\u89c2\u770b\u7684\u7528\u6237\u7ec4\uff08\u53ef\u591a\u9009\uff09'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-09-03 09:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        ('bee_django_wiki', '0003_auto_20190903_1650'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='topic',
            name='view_group',
        ),
        migrations.AddField(
            model_name='topic',
            name='view_group',
            field=models.ManyToManyField(to='auth.Group'),
        ),
    ]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'

from django.conf.urls import include, url
from . import views

app_name = 'bee_django_wiki'
urlpatterns = [
    url(r'^test$', views.test, name='test'),
    url(r'^$', views.TopicList.as_view(), name='index'),
    # 分类
    url(r'^classify/list$', views.ClassifyList.as_view(), name='classify_list'),
    url(r'^classify/add$', views.ClassifyCreate.as_view(), name='classify_add'),
    url(r'^classify/update/(?P<pk>[0-9]+)$', views.ClassifyUpdate.as_view(), name='classify_update'),
    url(r'^classify/delete/(?P<pk>[0-9]+)$', views.ClassifyDelete.as_view(), name='classify_delete'),
    # 主题
    url(r'^topic/list$', views.TopicList.as_view(), name='topic_list'),
    url(r'^topic/detail/(?P<pk>[0-9]+)$', views.TopicDetail.as_view(), name='topic_detail'),
    url(r'^topic/add/$', views.TopicCreate.as_view(), name='topic_add'),
    url(r'^topic/update/(?P<pk>[0-9]+)$', views.TopicUpdate.as_view(), name='topic_update'),
    url(r'^topic/delete/(?P<pk>[0-9]+)$', views.TopicDelete.as_view(), name='topic_delete'),
    url(r'^topic/upload/image$', views.upload_image, name='upload_image'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-09-03 08:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_wiki', '0002_auto_20190903_1645'),
    ]

    operations = [
        migrations.AlterField(
            model_name='topic',
            name='tag',
            field=models.CharField(blank=True, max_length=180, null=True, verbose_name='\u6807\u7b7e'),
        ),
    ]
